import * as i0 from '@angular/core';
import { createNgModule, Injectable, InjectionToken, NgModule, Optional, Inject } from '@angular/core';
import { ROUTES } from '@angular/router';

class LazyModuleLoaderBase {
    constructor() {
        this.toLoad = new Map();
        this.loading = new Map();
    }
    toLoadKeys() {
        return Array.from(this.toLoad.keys());
    }
    add(lazyRoutes) {
        lazyRoutes.forEach(routes => routes.forEach(({ name, loadChildren }) => {
            if (!this.loading.has(name)) {
                this.toLoad.set(name, loadChildren);
            }
        }));
    }
}

class LazyModuleLoader extends LazyModuleLoaderBase {
    constructor(moduleRef) {
        super();
        this.moduleRef = moduleRef;
    }
    async load(name) {
        if (this.loading.has(name)) {
            return this.loading.get(name);
        }
        const moduleLoader = this.toLoad.get(name);
        if (!moduleLoader) {
            throw new Error(`module not found:${name}`);
        }
        const loadedAndRegistered = moduleLoader()
            .then(module => {
            this.toLoad.delete(name);
            return createNgModule(module, this.moduleRef.injector);
        })
            .catch(err => {
            this.loading.delete(name);
            return Promise.reject(err);
        });
        this.loading.set(name, loadedAndRegistered);
        return loadedAndRegistered;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: LazyModuleLoader, deps: [{ token: i0.NgModuleRef }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: LazyModuleLoader, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: LazyModuleLoader, decorators: [{
            type: Injectable,
            args: [{ providedIn: 'root' }]
        }], ctorParameters: function () { return [{ type: i0.NgModuleRef }]; } });

const LAZY_ROUTES_TOKEN = new InjectionToken('LAZY_ROUTES_TOKEN');

function matcher() {
    return null;
}

class LazyModule {
    constructor(lazyModuleLoader, lazyModuleRoutes) {
        if (lazyModuleRoutes) {
            lazyModuleLoader.add(lazyModuleRoutes);
        }
    }
    static register(routes) {
        return {
            ngModule: LazyModule,
            providers: [
                {
                    provide: LAZY_ROUTES_TOKEN,
                    useValue: routes,
                    multi: true
                },
                {
                    provide: ROUTES,
                    useValue: routes.map(d => ({ ...d, matcher })),
                    multi: true
                }
            ]
        };
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: LazyModule, deps: [{ token: LazyModuleLoader }, { token: LAZY_ROUTES_TOKEN, optional: true }], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "16.0.0", ngImport: i0, type: LazyModule }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: LazyModule }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: LazyModule, decorators: [{
            type: NgModule
        }], ctorParameters: function () { return [{ type: LazyModuleLoader }, { type: undefined, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [LAZY_ROUTES_TOKEN]
                }] }]; } });

/**
 * Generated bundle index. Do not edit.
 */

export { LAZY_ROUTES_TOKEN as AIAO_LAZY_ROUTES_TOKEN, LazyModule as AiaoLazyModule, LazyModuleLoader as AiaoLazyModuleLoader, LazyModuleLoaderBase as AiaoLazyModuleLoaderBase };
//# sourceMappingURL=aiao-lazy-module.mjs.map
